/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/*global jQuery, require */
/*jslint browser: true, maxerr: 50, indent: 4 */
var Mercurial = (function ($) {
	'use strict';

	var mercurial, update, log;

	mercurial = {
		init : function () {
			var repositoryInfoColumn = $('td.repository-info');

			$.map(repositoryInfoColumn, function (column) {
				mercurial.getRepositoryInfo(column, 0);
			});

			$('a.repository-reload').bind('click', mercurial.refreshRepositoryStatus);

			if ($('table.queue').length > 0) {
				mercurial.updateRepositories();
			}
		},

		refreshRepositoryStatus : function (event) {
			event.preventDefault();

			var repositoryID, column;

			repositoryID = $(this).attr('rel');
			column       = $('td.repository-info[rel="' + repositoryID + '"]');

			mercurial.getRepositoryInfo(column, 1);
		},

		updateRepositories : function () {
			// Define function vars
			var repositories;

			// Get repositories
			mercurial.repositories = $('input[name="queue[]"]');
			mercurial.options      = $('input[name="options[]"]').map(function () {
				return $(this).val();
			}).get();

			// Interact if at least one repository is available
			if (mercurial.repositories.length > 0) {
				mercurial.updateRepository();
			}
		},

		updateRepository : function () {
			// Define function vars
			var currentItem, currentRow, repositoryID, queueStatus, options, params;

			// Element settings
			currentItem  = $(mercurial.repositories[0]);
			repositoryID = currentItem.val();
			queueStatus  = $('td.queuestatus', 'tr[rel="queue-' + repositoryID + '"]');

			// Suggest processing
			queueStatus.addClass('processing');
			queueStatus.parent().css('display', 'table-row');

			// Set payload params
			params = {
				page         : 'mercurial_ajax',
				func         : 'updateRepository',
				repositoryID : repositoryID,
				options      : mercurial.options
			};

			// Update repository
			mercurial.ajax('post', 'json', 'index.php', params, function (data) {
				// Remove processing class
				queueStatus.removeClass('processing');

				// Processed queue handling
				if (data.status === 'error') {
					queueStatus.append('<p class="error">' + data.message + '</p>');
					queueStatus.next().addClass('error');
				} else {
					if (data.message.clean) {
						queueStatus.append('<p class="clean">' + data.message.clean + '</p>');
						queueStatus.next().addClass('clean');
					}

					if (data.message.changes) {
						queueStatus.append('<p class="changes">' + data.message.changes + '</p>');
						queueStatus.next().addClass('changes');

						if (data.message.updates) {
							queueStatus.append('<p class="updates">' + data.message.updates + '</p>');
						}
					}
				}

				// Remove processed repository
				mercurial.repositories.splice(0, 1);

				// Process next item in queue
				if (mercurial.repositories.length > 0) {
					mercurial.updateRepository();
				}
			});
		},

		// Get repository info
		getRepositoryInfo : function (column, forceCache) {
			var repositoryID, params, tags, incoming, outgoing, status, summary;

			repositoryID = $(column).attr('rel');
			params       = {
				page         : 'mercurial_ajax',
				func         : 'getRepositoryInfo',
				repositoryID : repositoryID,
				forceCache   : forceCache
			};

			$(column).empty().addClass('loading');

			mercurial.ajax('post', 'json', 'index.php', params, function (data) {
				tags       = data.tags;
				incoming   = data.incoming;
				outgoing   = data.outgoing;
				status     = data.status;
				summary    = data.summary;

				$('<ul/>').addClass('repository-status').attr('rel', repositoryID).appendTo(column);

				mercurial.createTagInfo(column, repositoryID, tags, summary);
				mercurial.createStatusInfo(column, repositoryID, incoming, 'incoming');
				mercurial.createStatusInfo(column, repositoryID, outgoing, 'outgoing');
				mercurial.createStatusInfo(column, repositoryID, status, 'status');

				$(column).removeClass('loading');
			});
		},

		createTagInfo : function (column, repositoryID, data, summary) {
			var tags, subsequent, changeset, link, container, content;

			tags       = data.tags;
			subsequent = data.subsequent;
			changeset  = data.changeset;
			link       = $('<a/>').attr({ 'href' : 'index.php?page=mercurial&func=update&id=' + repositoryID, 'title' : summary.user + ' (' + summary.date + '): ' + summary.summary });

			if (tags !== undefined) {
				$.map(tags, function (tag) {
					container = $('<span/>').addClass('iconized tag');
					content   = $('<span/>').addClass('iconized-content');

					content.html(tag + ' +' + subsequent);
					content.appendTo(container);
					container.appendTo(link);
					link.appendTo(column);
				});
			} else {
				container = $('<span/>').addClass('iconized changeset');
				content   = $('<span/>').addClass('iconized-content');

				content.html(changeset + ' +' + subsequent);
				content.appendTo(container);
				container.appendTo(link);
				link.appendTo(column);
			}
		},

		createStatusInfo : function (column, repositoryID, data, type) {
			var wrapper, holder, link, container, content;

			wrapper   = $('<li/>');
			holder    = $('ul.repository-status[rel="' + repositoryID + '"]');
			link      = $('<a/>').attr({ 'href' : 'index.php?page=mercurial&func=' + type + '&id=' + repositoryID, 'title' : type.ucfirst() });
			container = $('<span/>').addClass('iconized ' + type);
			content   = $('<span/>').addClass('iconized-content');

			if (data) {
				wrapper.addClass('affected');
			}

			content.html(data);
			content.appendTo(container);
			container.appendTo(link);
			link.appendTo(wrapper);
			wrapper.appendTo(holder);
		},

		initMercurialTree : function (selector) {
			$('div.' + selector).jstree({
				plugins : ['ui', 'themes', 'json_data'],
				core: {
					initially_open : ['#root'],
					animation      : false
				},
				themes: {
					url   : '/data/dyn/public/mercurial/js/jstree/themes/default/style.css',
					dots  : false,
					theme : 'default',
					icons : false
				},
				json_data : {
					ajax : {
						url : 'index.php?page=mercurial_ajax&func=getMercurialTree'
					}
				},
				ui : {
					select_multiple_modifier : 'off',
					select_range_modifier    : 'off'
				}
			}).bind('select_node.jstree', function (event, data) {
				$('input.' + selector + '_hidden').val(data.rslt.obj.attr('url'));
			});
		},

		ajax : function (type, datatype, url, data, callback) {
			$.ajax({
				type     : type,
				dataType : datatype,
				url      : url,
				data     : data,
				success  : callback
			});
		}
	};

	update = {
		toggleRevisionSelection : function () {
			var node, changeset;

			$('input.filterable').bind('change', function () {
				$('div.node').toggleClass('all');
				$('div.node').removeClass('selected');
				$('div.changesets').scrollTop(0);
			});

			$('div.node').bind('click', function () {
				node      = $(this);
				changeset = null;

				$('div.node').not(node).removeClass('selected');

				if (node.hasClass('selected')) {
					node.removeClass('selected');
				} else {
					node.addClass('selected');
					changeset = node.attr('rel');
				}

				$('input#changeset').val(changeset);
			});
		}
	};

	mercurial.Update = update;

	$.fn.exists = function () {
		return ($(this).length > 0);
	};

	String.prototype.ucfirst = function () {
		return this.charAt(0).toUpperCase() + this.slice(1);
	};

	log = function (args) {
		if (typeof (window.console) !== 'undefined') {
			window.console.log('Mercurial: ', arguments);
		}
	};

	return mercurial;

}(jQuery));

jQuery(function () {
	'use strict';

	Mercurial.init();
});
