<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command_Version
 */
class Hg_Command_Version extends Hg_Command implements Hg_Interface_Command {
	/**
	 * Command
	 *
	 * @var <string>
	 */
	protected $command = 'version';

	/**
	 * Service
	 *
	 * @var <Hg_Service>
	 */
	protected $service = null;

	/**
	 * Version
	 *
	 * @var <array>
	 */
	protected $version = array();

	/**
	 * Version pattern
	 *
	 * @var <string>
	 */
	protected $versionPattern = 'X.Y.Z';

	/**
	 * Raw version
	 *
	 * @var <boolean>
	 */
	protected $getVersionAsRaw = false;

	/**
	 * Required options
	 *
	 * @var <array>
	 */
	protected $requiredOptions = array(
		'noninteractive' => null
	);

	/**
	 * Allowed options
	 *
	 * @var <array>
	 */
	protected $allowedOptions = array(
		'output' => null
	);

	/**
	 * Constructor
	 *
	 * @param <mixed> $params Mixed parameters
	 * @param <Hg_Service> $service Instance of Hg_Service
	 */
	public function __construct(array $params = array(), Hg_Service $service) {
		$this->service = $service;
	}

	/**
	 * Execute command
	 *
	 * @param <array> $params Array of parameters
	 *
	 * @return <Hg_Command_Version> Instance of self
	 */
	public function execute(array $params = array()) {
		if (empty($params) === false) {
			$this->setOptions($params);
		}

		$this->addOptions(array(
			'noninteractive' => null
		));

		$this->setCommandString();

		exec($this->commandString, $this->output, $this->status);

		if ($this->status > 1) {
			throw new Hg_Exception('Command Line Error');
		}

		return $this;
	}

	/**
	 * Set pattern option
	 *
	 * @param <string> $pattern Pattern to set
	 *
	 * @return <Hg_Command_Version> Instance of self
	 */
	public function pattern($pattern) {
		$this->versionPattern = $pattern;

		return $this;
	}

	/**
	 * Set raw option
	 *
	 * @return <Hg_Command_Version> Instance of self
	 */
	public function raw() {
		$this->getVersionAsRaw = true;

		return $this;
	}

	/**
	 * Get status
	 *
	 * @return <int> Status return from executed command
	 */
	public function getStatus() {
		return $this->status;
	}

	/**
	 * Get command string
	 *
	 * @return <string> Command string which was executed
	 */
	public function getCommand() {
		return $this->commandString;
	}

	/**
	 * Get output
	 *
	 * @param <string> $type Type of the output
	 * @param <string> $format Format of the output
	 *
	 * @return <mixed> Formatted output by type and format
	 */
	public function getOutput($type = null, $format = null) {
		static $output = null;

		switch ($type) {
			case 'external':
				$output = new Hg_Output($this->output);
				$output->format($format);

				break;

			case 'internal':
				$internal = null;
				$version  = array();

				if (preg_match('!\(([^\)]+)\)!', $this->output[0], $version)) {
					$version['raw'] = trim(str_replace('version', '', $version[1]));

					if ($this->getVersionAsRaw) {
						$internal = $version['raw'];
					}
					else {
						if (strstr($version['raw'], '+')) {
							$ver_parts = explode('+', $version['raw']);

							if (date_parse($ver_parts[1])) {
								$version['date'] = $ver_parts[1];
							}
								else {
									$version['changeset'] = $ver_parts[1];
								}
						}
							else {
								$ver_parts[0] = $version['raw'];
							}

						$version_tmp = explode('.', $ver_parts[0]);

						$this->version['major']       = $version_tmp[0];
						$this->version['minor']       = $version_tmp[1];
						$this->version['maintenance'] = $version_tmp[2];

						unset($version[0], $version[1]);

						$pattern = $this->versionPattern;
						$pattern = str_replace('X', $this->version['major'], $pattern);
						$pattern = str_replace('Y', $this->version['minor'], $pattern);
						$pattern = str_replace('Z', $this->version['maintenance'], $pattern);

						$internal = $pattern;
					}
				}

				$output = $internal;

				break;

			default:
				$output = $this->output;

				break;
		}

		return $output;
	}

	/**
	 * __toString Method
	 *
	 * @return <string> Output string
	 */
	public function __toString() {
		return $this->getOutput('internal');
	}
}
