<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Mercurial controller ajax
 */
class sly_Controller_Mercurial_Ajax extends sly_Controller_Mercurial implements sly_Controller_Interface {
	protected $init;

	/**
	 * Get mercurial tree
	 */
	public function getMercurialTreeAction() {
		$directory   = new Mercurial_Util_Directory();
		$directories = $directory->getBaseDirectories();

		$response = new sly_Response(json_encode($directories), 200);
		$response->setContentType('application/json', 'UTF-8');

		return $response;
	}

	/**
	 * Get repository info
	 */
	public function getRepositoryInfoAction() {
		$this->init();

		$repositoryID = sly_request('repositoryID', 'int', 0);
		$forceCache   = sly_request('forceCache', 'boolean', false);
		$cache        = Mercurial_Cache::getCache();
		$namespace    = 'dashboard.info';
		$key          = 'repository.' . $repositoryID;

		if ($forceCache) {
			$cache->delete($namespace, $key);
		}

		$info = $cache->get($namespace, $key);

		if (empty($info)) {
			$info['tags']     = $this->getRepositoryTags($repositoryID);
			$info['incoming'] = $this->getRepositoryIncoming($repositoryID);
			$info['outgoing'] = $this->getRepositoryOutgoing($repositoryID);
			$info['status']   = $this->getRepositoryStatus($repositoryID);
			$info['summary']  = $this->getRepositorySummary($repositoryID);

			$cache->set($namespace, $key, $info);
		}

		$response = new sly_Response(json_encode($info), 200);
		$response->setContentType('application/json', 'UTF-8');

		return $response;
	}

	/**
	 * Update repository
	 */
	public function updateRepositoryAction() {
		$this->init();

		// Get repository vars
		$repositoryID = sly_request('repositoryID', 'int', 0);
		$options      = sly_request('options', 'array', array());
		$cache        = Mercurial_Cache::getCache();
		$namespace    = 'dashboard.info';
		$key          = 'repository.' . $repositoryID;

		try {
			// Delete repository cache
			$cache->delete($namespace, $key);

			// Set repository vars
			$repository = Mercurial_Provider::getRepository($repositoryID);
			$lines      = $repository->setPull($options, 'default')->getOutput();

			// Iterate through
			foreach ($lines as $line) {
				// No changes
				if (preg_match('#^no\schanges\sfound#i', $line, $matches)) {
					$output['clean']   = 'Es wurden keine Änderungen gefunden.';
				}

				// Changesets, changes and files
				if (preg_match('#^added\s+(\d+)\s+changesets\s+with\s+(\d+)\s+changes\s+to\s+(\d+)\s+files$#i', $line, $matches)) {
					$output['changes'] = $matches[1] . ' Changesets mit ' . $matches[2] . ' Änderungen an ' . $matches[3] . ' Dateien hinzugefügt.';
				}

				// Updated, merged, removed and unresolved
				if (in_array('update', $options)) {
					if (preg_match('#^(\d+)\s+files\s+updated,\s+(\d+)\s+files\s+merged,\s+(\d+)\s+files\s+removed,\s+(\d+)\s+files\s+unresolved$#i', $line, $matches)) {
						$output['updates'] = $matches[1] . ' Dateien aktualisiert, ' . $matches[2] . ' zusammengeführt, ' . $matches[3] . ' entfernt und ' . $matches[4] . ' ungelöst.';
					}
				}
			}

			// Regenerate dashboard info
			$info['tags']     = $this->getRepositoryTags($repositoryID);
			$info['incoming'] = $this->getRepositoryIncoming($repositoryID);
			$info['outgoing'] = $this->getRepositoryOutgoing($repositoryID);
			$info['status']   = $this->getRepositoryStatus($repositoryID);
			$info['summary']  = $this->getRepositorySummary($repositoryID);

			// Set cache dashboard info
			$cache->set($namespace, $key, $info);

			// Success handling
			$response = new sly_Response(json_encode(
				array(
					'status'     => 'success',
					'repository' => $repositoryID,
					'message'    => $output
				)
			), 200);

			// Set response content type
			$response->setContentType('application/json', 'UTF-8');

			// Return response
			return $response;
		}
		catch (Hg_Exception $exception) {
			// Error handling
			$response = new sly_Response(json_encode(
				array(
					'status'     => 'error',
					'repository' => $repositoryID,
					'message'    => $exception->getMessage()
				)
			), 200);

			// Set response content type
			$response->setContentType('application/json', 'UTF-8');

			// Return response
			return $response;
		}
	}

	/**
	 * Get current tag
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <array> Tag infos
	 */
	protected function getRepositoryTags($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);
		$changeset  = $repository->getCurrentChangeset();
		$data       = $repository->getCurrentTags();
		$tags       = array();

		foreach ($data as $tag) {
			if (empty($tag) || $tag === 'tip') {
				$tags['changeset'] = $changeset;
			}
			else {
				$tags['tags'][] = $tag;
			}

			$tags['subsequent'] = $repository->getCurrentSubsequent();
		}

		return $tags;
	}

	/**
	 * Get repository incoming
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Count of incoming changes
	 */
	protected function getRepositoryIncoming($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);
		$data       = $repository->getIncoming(array(), 'default', '{rev}')->getOutput();
		$incoming   = array();

		foreach ($data as $line) {
			if (preg_match('#^(\d+)$#i', $line, $match)) {
				$incoming[] = $match[1];
			}
		}

		return count($incoming);
	}

	/**
	 * Get repository outgoing
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Count of outgoing changes
	 */
	protected function getRepositoryOutgoing($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);
		$data       = $repository->getOutgoing(array(), '{rev}')->getOutput();
		$outgoing   = array();

		foreach ($data as $line) {
			if (preg_match('#^(\d+)$#i', $line, $match)) {
				$outgoing[] = $match[1];
			}
		}

		return count($outgoing);
	}

	/**
	 * Get repository file status
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Count of status changes
	 */
	protected function getRepositoryStatus($repositoryID) {
		static $cases = array(
			'modified',
			'added',
			'removed',
			'deleted',
			'unknown'
		);

		$repository = Mercurial_Provider::getRepository($repositoryID);
		$status     = array();

		foreach ($cases as $case) {
			$status[] = count($repository->getStatus(array($case))->getOutput());
		}

		return array_sum(array_filter($status));
	}

	/**
	 * Get repository summary
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Repository summary
	 */
	protected function getRepositorySummary($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);

		$summary['user']    = $repository->getCurrentUser();
		$summary['date']    = $repository->getCurrentDate();
		$summary['summary'] = $repository->getCurrentCommitMessage();

		return $summary;
	}

	/**
	 * Check permission
	 *
	 * @param <string> $action Action parameter
	 *
	 * @return <boolean> State if user has permission to access this controller
	 */
	public function checkPermission($action) {
		$user = sly_Util_User::getCurrentUser();
		return $user->isAdmin() || $user->hasRight('mercurial[]');
	}
}
